
/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// body.cpp Fri Jun 8 2001
// Description: Body is the player's physical interface to the world.
//							The body has senses, and is capable of actions.  The
//							Player asks the body for sensory information and
//							tells the body to commit actions.
//////////////////////////////////////////////////////////////////////

#include "body.h"
#include "errors.h"


Body::Body()
{
}

Body::~Body()
{

}

void Body::senseWorld()
{
  // adds new server SensorData objects into SensorDataStore Object
  // this happens only once per loop
  cmConnection.handleSignals(sdsSensoryInfo);
}

// getNextSensorData() returns the "most important" SensorData object
//	that was recently received from the server.
SensorData* Body::getNextSensorData()
{
   SensorData* psdImportantData = NULL;

   if (!sdsSensoryInfo.empty())
   {
   		psdImportantData = sdsSensoryInfo.front();
   		sdsSensoryInfo.pop_front();
   }

   return psdImportantData;
   		
}

// isConnected checks to see if the body is still connected to the server
bool Body::isConnected()
{
	// TODO: perhaps, this isn't the best way to know if the body is connected to the server?
	return cmConnection.isServerAlive();
}

bool Body::hasSensorData()
{

	return (!sdsSensoryInfo.empty());
}

// _idAboutMe is modified and filled out by the server's checkin response
void Body::init(IdentityData &_idAboutMe, const char* _pcHostName = DEFAULT_HOST, int _iPort = PORT, int _iReconnectNumber = 0)
{
	cmConnection.init(_idAboutMe, _pcHostName, _iPort, _iReconnectNumber);
}

int Body::execute(const Action* _pacAction)
{
	ServerCommand* pscCmd = _pacAction->translate();
	
	// send the servercommand to the server
	cmConnection << *pscCmd;
	
	// delete the command
	delete pscCmd;
	
	return 1;
}

int Body::execute(const ActionList& alActions)
{
	// this function executes a list of actions... letting server commands
	// that are "free" (such as turnneck and say) pass through, while only
	// letting through one command that costs something (like dash, turn, and kick)

	alTheList = alActions;
	return execute();
		
//	Action* pacAction;
//	ServerCommand* pscCmd;
//	unsigned int i;							// unsigned int for an index (alActions.size() is unsigned)
//
//	// for this version limit the commands sent to only one turn, dash, or kick
//	// + many turnnecks, says... in the future a command or action queue might be
//	// useful... with a new action called clearqueue... that way the client could do
//	// planning
//
//	bool fSentOneDuration = false;
//	
//	// TODO: actually use error system/implement properly
//	for(i = 0; i < alActions.size(); i++)
//	{
//		pacAction = alActions.front();		
//		alActions.pop_front();
//		if (pacAction == NULL) continue;
//
//       	pscCmd = pacAction->translate();
//       	if (pscCmd == NULL) continue;
//
//       	if (pscCmd->getDuration() == 1)
//        {
//          	// send the servercommand to the server 	
//           	// INSPECT("sending... " << *pacAction << ", " << typeid(*pscCmd).name());
//   		    cmConnection << *pscCmd;
//       	}
//       	else if ((pscCmd->getDuration() > 0) && !fSentOneDuration)
//      	{
//       		fSentOneDuration = true;
//       	
//           	// send the servercommand to the server 	
//           	// INSPECT("sending... " << *pacAction << ", " << typeid(*pscCmd).name());
//  	    	cmConnection << *pscCmd;
//       	}
//
//       	// delete the command
//       	//delete pscCmd;
//        return pscCmd;
//	}
//
//	return NULL; // NO_ERROR;
}

int Body::execute()
{
    // -> this function makes the assumption that the local actionlist
    // contains all the 0 duration actions at the front
    // -> this function executes any free actions on a certain actionlist
    // upon its first call, and then a single action with a duration of 1
    // subsequent calls will execute another action with a duration of 1
    // an empty list results in no action

    int iNumActionsExecuted = 0;

    if(alTheList.empty())
      return iNumActionsExecuted; // EMPTY_LIST
    
    Action* pacAction;
    
    while(!alTheList.empty())
      if(alTheList.front() != NULL)
        {
	  pacAction = alTheList.front();
	  alTheList.pop_front();
	  int iDuration = pacAction->getDuration();
	  ServerCommand* sc = pacAction->translate();
	  INSPECT ("Sending action: " << *pacAction);
	  cmConnection << *sc;
	  delete sc;
	  delete pacAction;
	  iNumActionsExecuted++;
	  // quit after we execute an action with the cost of 1
	  if(iDuration == 1)
	    {
	      INSPECT ("Sent action, quitting");
	      break;
	    }
        }
      else
	alTheList.pop_front(); // pops NULL element (how did it get there kids?)
    
    return iNumActionsExecuted;
}
