/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// angle.cpp Mon Jun 18 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "angle.h"
#include "sensor.h"

Angle::Angle() : v( 0.0 )
{}

Angle::Angle( double _v ) : v( mod2pi( _v ) )
{}

// conversion to double; provides comparison operators <,>,<=,>=,==,!=

Angle::operator double() const
{
    return v;
}

// arithmetic (+,-,*,/)

Angle Angle::operator - ( ) const
{
    return Angle( -v );
}

Angle &Angle::operator += ( const Angle &ang )
{
    v = mod2pi( v + ang.v );
    return *this;
}

Angle &Angle::operator -= ( const Angle &ang )
{
    v = mod2pi( v - ang.v );
    return *this;
}

Angle &Angle::operator /= ( double s )
{
    v = mod2pi( v / s );
    return *this;
}

Angle &Angle::operator *= ( double s )
{
    v = mod2pi( v * s );
    return *this;
}

Angle Angle::operator + ( const Angle &ang ) const
{
    return Angle( v + ang.v );
}

Angle Angle::operator - ( const Angle &ang ) const
{
    return Angle( v - ang.v );
}

Angle Angle::operator / ( double s ) const
{
    return Angle( v / s );
}

Angle Angle::operator * ( double s ) const
{
    return Angle( v * s );
}

// just for writing a + 2 * b instead of a + b * 2
// friend
Angle operator * ( double s, const Angle &b )
{
    return Angle( s * b.v );
}

// shift an angle into the intervall [-pi, pi]
double Angle::mod2pi( double angle ) // used to be static
{
		if (!IsDirValid(angle))
		{
			ALERT("Attempted to create an unrealistic angle, returning an Angle of 0rad.");
			return 0;
		}
		
		angle = fmod( angle, 2*M_PI );
			
    if ( angle > M_PI )
        angle -= 2.0 * M_PI;
    if ( angle <= -M_PI )
        angle += 2.0 * M_PI;

    return angle;
}


// ******************************************** Utility functions *************
// conversion RAD -> DEG

double RAD_TO_DEG( double ang )
{
    return double( ang ) * 180.0 / M_PI;
}


// conversion DEG -> RAD

Angle DEG_TO_RAD( double ang )
{
    return Angle( ang * M_PI / 180.0 );
}


// Angle between the vectors a and b

Angle angleBetween( const Vector2d &a, const Vector2d &b )
{
    return b.getAngle() - a.getAngle();
}


// Bisecting line/angle between two angles

Angle bisect( Angle angle1, Angle angle2 )
{
    return angle1 + 0.5 * ( angle2 - angle1 );
}


ostream& operator<<(ostream& out, const Angle& a)
{
    out << RAD_TO_DEG( a ) << "^";

    return out;
}