/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// angle.h Mon Jun 18 2001
// Description:
//////////////////////////////////////////////////////////////////////

#ifndef ANGLE_H
#define ANGLE_H

#include <iostream.h>
#include <math.h>
#include "constants.h"
#include "minmax.h"

// Class for angles in the interval [-pi, pi].
// This is the kind of angles the soccerserver uses.

class Vector2d;

#include "vector2d.h"


class Angle
{
public:

    Angle();
    Angle( double _v );

    // conversion to double; provides comparison operators <,>,<=,>=,==,!=
    operator double() const;

    // arithmetic (+,-,*,/)

    Angle operator - ( ) const;

    Angle &operator += ( const Angle &ang );

    Angle &operator -= ( const Angle &ang );

    Angle &operator /= ( double s );

    Angle &operator *= ( double s );

    Angle operator + ( const Angle &ang ) const;

    Angle operator - ( const Angle &ang ) const;

    Angle operator / ( double s ) const;

    Angle operator * ( double s ) const;

    // just for writing a + 2 * b instead of a + b * 2
    friend Angle operator * ( double s, const Angle &b );


private:

    double v;

    // shift an angle into the intervall [-pi, pi]
    double mod2pi( double angle ); // used to be static inline
}
; // end of class Angle



const Angle PI = M_PI;
const Angle PI_2 = M_PI_2;
const Angle PI_3 = Angle( M_PI / 3.0 );
const Angle PI_4 = M_PI_4;
const Angle PI_6 = Angle( M_PI / 6.0 );
const Angle PI_8 = Angle( M_PI / 8.0 );

// conversion RAD -> DEG
double RAD_TO_DEG( double ang );

// conversion DEG -> RAD
Angle DEG_TO_RAD( double ang );

// Angle between the vectors a and b
Angle angleBetween( const Vector2d &a, const Vector2d &b );

// Bisecting line/angle between two angles
Angle bisect( Angle angle1, Angle angle2 );

ostream& operator<<(ostream& out, const Angle& a);

#endif
