/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// types.h Thu Jun 7 2001
// Description: This file includes custom enumerations and structs
//							that define different types of data about the player
//							and the field
//////////////////////////////////////////////////////////////////////

#ifndef _TYPES_H_
#define _TYPES_H_

// Enumeration of line types

enum line_type {
	LINE_UNKNOWN = -1,	// "anonymous" line, used in sensor data
				// i.e. we see a line but don't know which one it is
	LINE_TOP = 0,
	LINE_LEFT,
	LINE_RIGHT,
	LINE_BOTTOM,
	NUM_LINES		// Number of lines (excluding the anonymous line)
};

enum view_width {
	VIEW_WIDTH_UNKNOWN = -1,
  VIEW_WIDTH_NARROW,
  VIEW_WIDTH_NORMAL,
  VIEW_WIDTH_WIDE
};

enum view_quality {
	VIEW_QUALITY_UNKNOWN = -1,
  VIEW_QUALITY_LOW,
  VIEW_QUALITY_HIGH
};

enum playing_side {
  SIDE_UNKNOWN = -1,
  SIDE_LEFT,
  SIDE_RIGHT
};

enum play_mode {
  PLAY_MODE_UNKNOWN = -1,
  PLAY_MODE_BEFORE_KICK_OFF = 0,
  PLAY_MODE_TIME_OVER,
  PLAY_MODE_PLAY_ON,
  PLAY_MODE_KICK_OFF_US,
  PLAY_MODE_KICK_OFF_THEM,
  PLAY_MODE_KICK_IN_US, // 5
  PLAY_MODE_KICK_IN_THEM,
  PLAY_MODE_FREE_KICK_US,
  PLAY_MODE_FREE_KICK_THEM,
  PLAY_MODE_CORNER_KICK_US,
  PLAY_MODE_CORNER_KICK_THEM, // 10
  PLAY_MODE_GOAL_KICK_US,
  PLAY_MODE_GOAL_KICK_THEM,
  PLAY_MODE_GOAL_US,
  PLAY_MODE_GOAL_THEM,
  PLAY_MODE_DROP_BALL, // 15
  PLAY_MODE_OFFSIDE_US,
  PLAY_MODE_OFFSIDE_THEM,
  PLAY_MODE_HALF_TIME,
  PLAY_MODE_TIME_EXTENDED,
  PLAY_MODE_GOALIE_CATCH_US, // 20
  PLAY_MODE_GOALIE_CATCH_THEM
};

// Enumeration of goal types ("left" and "right" refers to the
// absolute position on the field)
enum goal_type {
	GOAL_UNKNOWN = -1,	// "anonymous" goal
				// i.e. we see a goal but don't know which one it is
	GOAL_LEFT = 0,
	GOAL_RIGHT,
	NUM_GOALS		// Number of goals (excluding the anonymous goal)
};

// Enumeration of flag types
enum flag_type {
	FLAG_UNKNOWN = -1,	// "anonymous" flag, used in sensor data
				// i.e. we see a flag but don't know which one it is
	FLAG_TOP_LEFT = 0,
	FLAG_TOP_CENTER,
	FLAG_TOP_RIGHT,
	FLAG_BOTTOM_LEFT,
	FLAG_BOTTOM_CENTER,
	FLAG_BOTTOM_RIGHT,

	FLAG_P_LEFT_TOP,	// flags in penalty area
	FLAG_P_LEFT_CENTER,
	FLAG_P_LEFT_BOTTOM,
	FLAG_P_RIGHT_TOP,
	FLAG_P_RIGHT_CENTER,
	FLAG_P_RIGHT_BOTTOM,

	FLAG_CENTER,

	FLAG_GOAL_LEFT_TOP,	// goal posts
	FLAG_GOAL_LEFT_BOTTOM,
	FLAG_GOAL_RIGHT_TOP,
	FLAG_GOAL_RIGHT_BOTTOM,

	FLAG_TOP_LEFT_50,	// flags outside of field
	FLAG_TOP_LEFT_40,
	FLAG_TOP_LEFT_30,
	FLAG_TOP_LEFT_20,
	FLAG_TOP_LEFT_10,
	FLAG_TOP_0,
	FLAG_TOP_RIGHT_10,
	FLAG_TOP_RIGHT_20,
	FLAG_TOP_RIGHT_30,
	FLAG_TOP_RIGHT_40,
	FLAG_TOP_RIGHT_50,

	FLAG_BOTTOM_LEFT_50,
	FLAG_BOTTOM_LEFT_40,
	FLAG_BOTTOM_LEFT_30,
	FLAG_BOTTOM_LEFT_20,
	FLAG_BOTTOM_LEFT_10,
	FLAG_BOTTOM_0,
	FLAG_BOTTOM_RIGHT_10,
	FLAG_BOTTOM_RIGHT_20,
	FLAG_BOTTOM_RIGHT_30,
	FLAG_BOTTOM_RIGHT_40,
	FLAG_BOTTOM_RIGHT_50,

	FLAG_LEFT_TOP_30,
	FLAG_LEFT_TOP_20,
	FLAG_LEFT_TOP_10,
	FLAG_LEFT_0,
	FLAG_LEFT_BOTTOM_10,
	FLAG_LEFT_BOTTOM_20,
	FLAG_LEFT_BOTTOM_30,

	FLAG_RIGHT_TOP_30,
	FLAG_RIGHT_TOP_20,
	FLAG_RIGHT_TOP_10,
	FLAG_RIGHT_0,
	FLAG_RIGHT_BOTTOM_10,
	FLAG_RIGHT_BOTTOM_20,
	FLAG_RIGHT_BOTTOM_30,

	NUM_FLAGS		// Number of flags (excluding the anonymous flag)
};

enum possession_state {
  PS_UNKNOWN = -1,
  PS_OURS,
  PS_THEIRS,
  PS_FREE
};

// error types returned by this module

enum comm_error {
  COMM_OK = 0,
  COMM_ERROR,	           	// Unspecified error
  COMM_ERROR_SERVER_NOT_FOUND,  // Server name not found
  COMM_ERROR_SOCKET_INIT,	// Couldn't init socket
  COMM_ERROR_SERVER_INIT,	// Server didn't accept our init command
  COMM_ERROR_NO_DATA,	        // Server has no new data
  COMM_ERROR_PARSE,	        // Server sent us something we could not parse
  COMM_ERROR_SIGNAL_INIT,       // initing the signal handlers failed
};

// from sensor.h
// types of clients that can check into the server
enum client_type {
  CLIENT_UNKNOWN = -1,
  CLIENT_GOALIE,
  CLIENT_FIELDPLAYER,
  CLIENT_ONLINECOACH,
  CLIENT_TRAINCOACH
};

enum sensor_data_type {
  SENSOR_TYPE_SEE,
  SENSOR_TYPE_HEAR,
  SENSOR_TYPE_SENSE_BODY,
  SENSOR_TYPE_EXACT,
  SENSOR_TYPE_OK,
  SENSOR_TYPE_ERROR
};

enum team_number {
  TEAM_UNKNOWN = -1,
  TEAM_US,
  TEAM_THEM
};

enum goalie_player {
  GOALIE_UNKNOWN = -1,
  GOALIE_TRUE,
  GOALIE_FALSE
};

enum sender_type {
  SENDER_SELF = 0,
  SENDER_REFEREE,
  SENDER_DIR,
  SENDER_PLAYER,
  SENDER_ONLINECOACH,
  SENDER_COACH
};

// structure for synchronous player commands for one simulation cycle
// see Soccerserver Manual Appendix A2
// (asynchronous commands are sent directly using their own send-methods)

enum CycleCommand {
  NO_COMMAND =  0,   // param1 = param2 = 0
  KICK_COMMAND,      // param1 = power, param2 = dir_rad
  DASH_COMMAND,      // param1 = power, param2 = 0
  TURN_COMMAND,      // param1 = moment_rad, param2 = 0
  CATCH_COMMAND,     // for goalie only, param1 = dir_rad, par2 = 0
  MOVE_COMMAND       // pm_before_kick_off only, param1=x, param2=y
};

#endif
