/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// behindballbehavior.cpp Fri Jul 13 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "behindballbehavior.h"

BehindBallBehavior::BehindBallBehavior(const WorldModel &_wmWorld): perBehindBall(_wmWorld)
{
}

BehindBallBehavior::~BehindBallBehavior()
{
}

bool BehindBallBehavior::isUseful()
{
  if (!perBehindBall.isActiveMode())
    return false;
  
  if ( ((perBehindBall.getPossession() != PS_OURS) || (perBehindBall.inOurPenaltyArea(perBehindBall.getBall())))
       && !perBehindBall.isWellBehindBall() 
       && !perBehindBall.amIClosestToBall()
       && (perBehindBall.getBallAge() < 15) 
       && !perBehindBall.ballTooDeep() )
    return true;
  
  return false;
}

void BehindBallBehavior::evaluate(RecommendationList& recList)
{
	//create the Recommendation to be passed back
	BehindBallRecommendation bbrBehindBallRec;
	bbrBehindBallRec.szBehaviorName = "behindball";

	int iGrade = 0;

	iGrade = (int)(perBehindBall.distToLine() * 1.5);
	//set grade depending on where the ball is
	if (perBehindBall.inAttackingThird (perBehindBall.getBall()))
	  iGrade += 20;
	else if (perBehindBall.inMiddleThird (perBehindBall.getBall()))
	  iGrade += 25;
	else if (perBehindBall.inOurPenaltyArea (perBehindBall.getBall()))
	  iGrade += 35;
	else if (perBehindBall.inDefensiveThird (perBehindBall.getBall()))
	  iGrade += 30;
	
	if (iGrade > 70)
	  iGrade = 70;
	else if (iGrade <= 40)
	  iGrade = 0;
	
	  if ( (iGrade < 60) && (!perBehindBall.inOurPenaltyArea (perBehindBall.getBall())) )
	    iGrade = 60;
	
	if(perBehindBall.lowStamina())
	  iGrade = 70;
	
	bbrBehindBallRec.setGrade(iGrade);

	//set the action of the behind ball move and return recommendation
	Angle aRunAngle = PI - perBehindBall.getBodyDir();

	RunAction raTemp;
	if (iGrade > 65)
	  raTemp.setIntensity(85);
	else
	  raTemp.setIntensity(85);
	
	if(perBehindBall.slowRun())
	  raTemp.setIntensity(15);

	raTemp.setDirection(aRunAngle);
	bbrBehindBallRec.addAction ( raTemp );
	raTemp.setDirection(0);
	bbrBehindBallRec.addAction ( raTemp );
	
    recList.insert(bbrBehindBallRec);


}
