/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// catchbehavior.cpp Mon Jul 23 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "catchbehavior.h"

#undef TRACE
#define TRACE(cmd)
//#define TRACE(message) cout << __FILE__ << ":" << __LINE__ << ":" << perCatch.getTime()  << ": " << message << endl;

CatchBehavior::CatchBehavior(const WorldModel& _wmWorld) : perCatch(_wmWorld)
{
    iLastCatchTime = UNKNOWN;
}

CatchBehavior::~CatchBehavior()
{
}

bool CatchBehavior::isUseful()
{
    // seems bad to abort catching, but if you don't know where the ball is, the catch will fail
    if((perCatch.getTime() - iLastCatchTime) <= CATCH_BAN_CYCLE)
        return false;

    if(perCatch.isBallOld())
        return false;

    if(!perCatch.isActiveMode())
        return false;

    if( !perCatch.inOurPenaltyArea() )
        return false;

    if(perCatch.isCatchable())
    {
        TRACE("Ball is catchable");
        return true;
    }

    return false;
}

void CatchBehavior::evaluate(RecommendationList& recList)
{
  Recommendation recFinal;
  recFinal.szBehaviorName = "catch";	
  
  //    if(!perCatch.isCatchable())
  //        return;
  
  TRACE("Catching");
  
  CatchAction caCatch;
  
  caCatch.setDirection( perCatch.catchAngle() );
  
  recFinal.addAction( caCatch );
  iLastCatchTime = perCatch.getTime();
  
  // eventually we will want to do catch probability calculations here
  recFinal.setGrade(100);
  
  recList.insert(recFinal);
}
