/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// centerbehavior.cpp Fri Jul 6 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "centerbehavior.h"

CenterBehavior::CenterBehavior(const WorldModel &_wmWorld): perCenter(_wmWorld)
{
}

CenterBehavior::~CenterBehavior()
{
}

bool CenterBehavior::isUseful()
{
  if (!perCenter.isActiveMode())
    return false;
  
  bool f = false;
  bool g = false;
  //should evaluate unless our team has possession
  if ( (perCenter.getPossession() != PS_OURS) || (perCenter.inOurPenaltyArea (perCenter.getBall())) )
    f = true;
  
  if ( (!perCenter.amIClosestToBall()) && (perCenter.getBallAge() < 15) )
    g = true;
  
  if ( f && g )
    return true;
  
  else return false;
  
}

void CenterBehavior::evaluate(RecommendationList& recList)
{
  //create the Recommendation to be passed back
  CenterRecommendation rCenterRec;
  rCenterRec.szBehaviorName = "center";
  //sets the ball position aa the virtual center
  double dVirtCenter = ( perCenter.getBallAbsPos() ).getY();
  
  //if outside the penalty area width, the respective edge
  //becomes the virtual center
  if ( fabs(dVirtCenter) > 0.5 * PENALTY_AREA_WIDTH - 12.0 )
    if ( dVirtCenter > 0.0 )
      dVirtCenter = 0.5 * PENALTY_AREA_WIDTH - 12.0;
    else
      dVirtCenter = -0.5 * PENALTY_AREA_WIDTH + 12.0;
  
  //  if(perCenter.isGoalie()) 
    dVirtCenter = 0.0; // JM

  //find the maximum distance a player can be away from the virtual center
  double dMaxDist = (0.5 * FIELD_WIDTH) + (0.5 * PENALTY_AREA_WIDTH - 5.0);
  
  //find fraction to virtual center and get initial grade
  double dDistToVCenter = perCenter.distToVirtCenter ( dVirtCenter  );
  double dPercToVCenter = fabs (dDistToVCenter) / dMaxDist;
  int iGrade = static_cast<int> (dPercToVCenter * 80); //CENTERING_MULTIPLIER);
  
  //add the bonus depending on where the ball is and set grade
  if (perCenter.inAttackingThird (perCenter.getBall()))
    iGrade += 20; //ATTACKING_BONUS;
  else if (perCenter.inMiddleThird (perCenter.getBall()))
    iGrade += 25; //MIDDLE_BONUS;
  else if (perCenter.inOurPenaltyArea (perCenter.getBall()))
    iGrade += 40; //PENALTY_AREA_BONUS;
  else if (perCenter.inDefensiveThird (perCenter.getBall()))
    iGrade += 30; //DEFENSIVE_BONUS;
  
  if (iGrade > 69)
    iGrade = 69;
  else if (iGrade < 30)
    iGrade = 0;
  
  //	if ( (iGrade < 60) && (perCenter.inOurPenaltyArea (perCenter.getBall())) )
  //		iGrade = 60;
  
  rCenterRec.setGrade(iGrade);
  
  //set the action of the centering move and return recommendation
  Angle aRunAngle;
  if (dDistToVCenter > 0.0)
    {
      aRunAngle = - PI_2 - perCenter.getBodyDir();
    }
  else
    {
      aRunAngle = PI_2 - perCenter.getBodyDir();
    }
  
  RunAction raTemp;

  if (perCenter.inDefensiveThird (perCenter.getBall()))
    raTemp.setIntensity(75);
  else
    raTemp.setIntensity(50);
  
  raTemp.setDirection(aRunAngle);
  
  rCenterRec.addAction ( raTemp );
  raTemp.setDirection(0);
  rCenterRec.addAction ( raTemp );	
  
  recList.insert(rCenterRec);
}
