/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// dispersebehavior.cpp Wed Jun 20 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "dispersebehavior.h"
#include "runaction.h"

DisperseBehavior::DisperseBehavior(const WorldModel &_wmWorld): perDisperse(_wmWorld)
{
}

DisperseBehavior::~DisperseBehavior()
{
}

bool DisperseBehavior::isUseful()
{
  if (!perDisperse.isActiveMode())
    return false;
  
  //behavior is useful for offense and defense
  if ((perDisperse.getMyX() > - 0.5 * FIELD_LENGTH) 
      && (!perDisperse.amIClosestToBall()) 
      //      && (perDisperse.getBallAge() < 15) 
    ) return true;
  
  return false;
}

void DisperseBehavior::evaluate(RecommendationList& recList)
{
  
  
  //create the Recommendation to be passed back
  Recommendation rDisperseRec;
  rDisperseRec.szBehaviorName = "disperse";
  
  //figure out what the disperse distance should be
  double dDispDist;
  if (perDisperse.getPossession() == PS_OURS)
    dDispDist = OFFENSIVE_DISPERSE_DIST;
  else
    {
      double dFieldPerc =  ( (0.5 * FIELD_LENGTH) + (perDisperse.getMyX()) ) / FIELD_LENGTH;
      dDispDist = dFieldPerc * DEFENSIVE_DISPERSE_MULT;
    }
  
  if (dDispDist <  5.0 )
    dDispDist = 5.0;
  
  //figure out a vector with the positions of the players within disperse distance
  
  Vector2d vDisperseVec (0.0, 0.0);
  Vector2d vDist;
  double dDistMagnitude;
  
  for (int i = 0; i < NUM_TEAMMATES; ++i)
    {
      vDist = perDisperse.relPos(perDisperse.ourTeamDistIndex(i));
      if ((vDist.getLength() < dDispDist) && (vDist.getLength() != 0.0))
	{
	  dDistMagnitude = vDist.getLength();
	  vDisperseVec += ((vDist / dDistMagnitude) * (dDispDist - dDistMagnitude));
	}
    }
  
  // take the negative of the composite distance vector and set recommendation
  vDisperseVec = -vDisperseVec;
  
  perDisperse.checkVec(vDisperseVec);
  
  double dGrade = (vDisperseVec.getLength()  / dDispDist) * 140; // used to - 10 ???
  
  if (dGrade > 68)
    dGrade = 68;
  else if (dGrade < 30)
    dGrade = 0;
  
  rDisperseRec.setGrade(dGrade);
  
  Angle aDir = vDisperseVec.getAngle();
  
  RunAction raTemp;
  raTemp.setDirection(aDir);

  raTemp.setIntensity(50);
  
  rDisperseRec.addAction(raTemp);
  raTemp.setDirection(0.0);
  rDisperseRec.addAction(raTemp);
  
  recList.insert(rDisperseRec);
  return;
  
}











