/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// interceptballbehavior.cpp Tue Jul 3 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "goalieinterceptbehavior.h"
#include "turnneckaction.h"
#include "changeviewaction.h"

GoalieInterceptBehavior::GoalieInterceptBehavior(WorldModel &_wmWorld)
    : perIntercept(_wmWorld), raLastInterceptAction(_wmWorld), wmWorld(_wmWorld)
{
}

GoalieInterceptBehavior::~GoalieInterceptBehavior()
{
}


bool GoalieInterceptBehavior::isUseful()
{
  // this module is useful when i'm not at the ball, and I'm the closest person, and its an active mode

  bool nti = perIntercept.needToIntercept();
  bool iam = perIntercept.isActiveMode();
  
  if (nti && iam)
    {
      INSPECT ("Goalie intercept useful");
      return true;
    }
  
  INSPECT ("*** Goalie intercept NOT useful");
  return false;
}

void GoalieInterceptBehavior::evaluate(RecommendationList& recList)
{
  // raRun is now a private data member
  
  Recommendation recIntercept;
  recIntercept.szBehaviorName = "interceptball";		
  InterceptAction raRun(wmWorld);
  
  Vector2d vDash;
  int iTime = perIntercept.getDashVector(vDash);			
  
  INSPECT ("Predicting: " << iTime << " Vector: " << vDash);
  dIntensity = vDash.getLength(); // save for prev. case		    

  if (iTime < 3) {
    dIntensity = 60.0;
  }

  raRun.setIntensity (dIntensity);
  raRun.setDirection(perIntercept.getTurnMoment(vDash.getAngle()));	  
  
  if (raRun == raLastInterceptAction 
      || fabs(vDash.getAngle()) < (DEG_TO_RAD(7.0))) 
    // angle value guessed from replayer
    {
      
      // Now make sure we don't do the same command twice  
      //    			cerr << perIntercept.getTime() << ": sendDashOnly" << endl;
      raRun.setDirection(0);
    }
  else
    {
      raLastInterceptAction.setIntensity(raRun.getIntensity());
      raLastInterceptAction.setDirection(raRun.getDirection());
    }
  
  // take your time when you can
  if(perIntercept.specialSituation())
    {
      raRun.setIntensity(20);
    }

  if(perIntercept.isTooFarFromBall())
    {
      raRun.setIntensity(40);
    }
  
  recIntercept.addAction(raRun);		
  raRun.setDirection(0);

  if (iTime < 4)
    {	
      for (int i = 0; i < iTime; ++i)
	{
	  recIntercept.addAction (raRun);
	}
    }
  else
    {				
      recIntercept.addAction(raRun);
      recIntercept.addAction(raRun); 
      recIntercept.addAction(raRun);
    }						
  
  recIntercept.setGrade(85);
  recList.insert(recIntercept);
}
