/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// scanbehavior.cpp Mon Jul 2 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "scanbehavior.h"
#include "turnaction.h"
#include "turnneckaction.h"
#include "changeviewaction.h"

const int ScanBehavior::OLD_AGE = 5;

ScanBehavior::ScanBehavior(const WorldModel &_wmWorld) : perScan(_wmWorld)
{
  fTriedLooking = true;
}

ScanBehavior::~ScanBehavior()
{}

bool ScanBehavior::isUseful()
{
    int iWorldAge = perScan.getWorldAge();
	bool f = perScan.isNeckTwisted();
	bool g = (iWorldAge > 4); //!= 0);
	
    if (f || g)
    {
        return true;
    }

    fTriedLooking = false;
    return false;
}

void ScanBehavior::evaluate(RecommendationList& recList)
{
    Recommendation recScan;
    recScan.szBehaviorName = "scan";  //for debugging only

    int iWorldAge = perScan.getWorldAge();

    if(!perScan.correctViewMode(VIEW_WIDTH_NORMAL, VIEW_QUALITY_HIGH))
    {
        ChangeViewAction cvaNewView(VIEW_WIDTH_NORMAL, VIEW_QUALITY_HIGH);
        recScan.addAction(cvaNewView);
    }

    double dGrade = 69 * iWorldAge/ScanBehavior::OLD_AGE;
    if (dGrade > 69)
        dGrade = 69;

	if (iWorldAge == UNKNOWN || perScan.isKickOffMode())
	{
		// keep spinning
		TurnAction taTurn(PI_6);
		recScan.addAction(taTurn);
		recScan.setGrade(99);
		recList.insert(recScan);
		
		return;					
	}
	
	if (perScan.isLookingAtBall())  // if the player is looking where the ball used to be
	{
		// and he doesn't see the ball
		if (iWorldAge > 4)
		{
			//straighten the neck
			TurnNeckAction tnaTurnNeck(perScan.relNeckAngle(0.0));
			recScan.addAction(tnaTurnNeck);
		
			//dTurnDirection = perScan.turnDirection();
			// and begin to turn in circles
			//TurnAction taTurn(dTurnDirection * PI_6);
			TurnAction taTurn(PI_6);
			recScan.addAction(taTurn);
			fTriedLooking = true;
		}
		else // and he sees the ball
		{
			// straighten the neck
			TurnNeckAction tnaTurnNeck(perScan.relNeckAngle(0.0));
			recScan.addAction(tnaTurnNeck);
			fTriedLooking = false;
		}

	}
	// if you've started spinning and you still can't see the ball
	else if ((fTriedLooking) && (iWorldAge > 4))
	{
		// keep spinning
		TurnAction taTurn(PI_6);
		recScan.addAction(taTurn);		
	}
	else if (iWorldAge > 4)
	{
		// otherwise turn your neck to the last place you saw the ball
		Angle aBall = perScan.angleToBall();
		if ((perScan.relNeckAngle(aBall) > PI_2) || (perScan.relNeckAngle(aBall) < -PI_2))
		{
			TurnAction taTurn(aBall);
			recScan.addAction(taTurn);
		}
		else
		{
			TurnNeckAction tnaTurnNeck(perScan.relNeckAngle(aBall));
			recScan.addAction(tnaTurnNeck);
		}
		fTriedLooking = true;
	}
	else
	{
	  TRACE("uhhh...");
	  return;
	}
	
	recScan.setGrade(dGrade);
	recList.insert(recScan);
/*
    // there are only five states, repeat them if the ball is not found
    if (iState == 4)
        iState = 0;
    else
        iState++;

    int iWorldAge = perScan.getWorldAge();

    double dGrade = 94 * iWorldAge/ScanBehavior::OLD_AGE;
    if (dGrade > 94)
        dGrade = 94;

    // if the ball is not in play turn in circles
    if (perScan.isBallNotInPlay() || (iWorldAge == UNKNOWN))
    {
        TurnAction taTurn(-PI_4);
        //TurnAction* ptaTurn = new TurnAction(PI_4);
        recScan.addAction(taTurn);
        recScan.setGrade(dGrade);
    }
    else if (iWorldAge >= OLD_AGE)
    {
        if (iState == 0)
        {
            Angle aTurnNeck = perScan.relNeckAngle(PI_4);

            TurnNeckAction tnaTurnNeck(aTurnNeck);
            iLastNeckTime = perScan.getTime();

            recScan.addAction(tnaTurnNeck);

            recScan.setGrade(dGrade);
        }
        else if (iState == 1)
        {
            Angle aTurnNeck = perScan.relNeckAngle(PI_2);

            TurnNeckAction tnaTurnNeck(aTurnNeck);
            iLastNeckTime = perScan.getTime();

            recScan.addAction(tnaTurnNeck);

            recScan.setGrade(dGrade);
        }
        else if (iState == 2)
        {
            Angle aTurnNeck = perScan.relNeckAngle(-PI_4);

            TurnNeckAction tnaTurnNeck(aTurnNeck);
            iLastNeckTime = perScan.getTime();

            recScan.addAction(tnaTurnNeck);

            recScan.setGrade(dGrade);
        }
        else if (iState == 3)
        {
            Angle aTurnNeck = perScan.relNeckAngle(-PI_2);

            TurnNeckAction tnaTurnNeck(aTurnNeck);
            iLastNeckTime = perScan.getTime();

            recScan.addAction(tnaTurnNeck);
        }
        else if (iState == 4)
        {

            TurnAction taTurn(PI_3);
            recScan.addAction(taTurn);
            recScan.setGrade(dGrade);
        }
    }
    else
    {
        if ( perScan.isNeckTwisted() && ((perScan.getTime()-iLastNeckTime) < 2) )
        {
            TurnNeckAction tnaTurnNeck(perScan.relNeckAngle(0.0));
            //TurnAction taTurn(-perScan.relNeckAngle(0.0));

            recScan.addAction(tnaTurnNeck);
            //recScan.addAction(taTurn);
            aLastAngle = 0.0;
            recScan.setGrade(96);
        }
        else
        {
            // this should never happen, but just in case...
            recScan.setGrade(0);
        }
    }

    return recScan;
*/
}

