/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// scanperception.cpp Mon Jul 2 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "scanperception.h"

ScanPerception::ScanPerception(const WorldModel &_wmWorld) : Perception(_wmWorld)
{

}

ScanPerception::~ScanPerception()
{
}


int ScanPerception::getWorldAge() const
{
	WorldBall wbBall = wmWorld.getWorldBall();
	
	return wbBall.getAge();		
}

bool ScanPerception::isWorldUnknown() const
{
	WorldBall wbBall = wmWorld.getWorldBall();
	
	if (wbBall.getAge() == UNKNOWN)
	{
		return true;
	}
	
	return false;
}

Angle ScanPerception::angleToBall() const
{
	WorldBall wbBall = wmWorld.getWorldBall();
	Vector2d vRelPos = relPos(wbBall);
	
	return vRelPos.getAngle();
}

bool ScanPerception::isNeckTwisted() const
{
	ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
	
	return (cwpClientPlayer.getHeadDir() != 0.0);
}

Angle ScanPerception::relNeckAngle(const Angle& aAbsNeckPosition) const
{
	ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
	
	return (aAbsNeckPosition - cwpClientPlayer.getHeadDir());
}

bool ScanPerception::isBallNotInPlay() const
{
	play_mode pmMode = wmWorld.getPlayMode();
	return ((pmMode == PLAY_MODE_BEFORE_KICK_OFF) || (pmMode == PLAY_MODE_TIME_OVER));
}


int ScanPerception::getTime() const
{
    return wmWorld.getTime();
}

bool ScanPerception::isLookingAtBall() const
{
	WorldBall wbBall = wmWorld.getWorldBall();
	ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
	
	double dAngle = abs(relPos(wbBall).getAngle() - cwpClientPlayer.getHeadDir());
	
	if (dAngle < (M_PI/4.0))
		return true;
		
	return false;
}

double ScanPerception::turnDirection() const
{
	WorldBall wbBall = wmWorld.getWorldBall();

	Vector2d vPosition = relPos(wbBall);
	Vector2d vVelocity = wbBall.getVelocity();
	
	if (vVelocity.getAngle() > vPosition.getAngle())
	{
		return -1.0;
	}
	
	return 1.0;
}

bool ScanPerception::isKickOffMode() const
{
		play_mode pmTemp = wmWorld.getPlayMode();
    if( (pmTemp == PLAY_MODE_BEFORE_KICK_OFF ) ||
        (pmTemp == PLAY_MODE_GOAL_US) ||
        (pmTemp == PLAY_MODE_GOAL_THEM) ||
        (pmTemp == PLAY_MODE_HALF_TIME) )
        return true;

    return false;
}
