/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// baseworldmodel.cpp Thu Jun 28 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "baseworldmodel.h"
#include "lowlevelcomm.h"

//int BaseWorldModel::count = 0;

BaseWorldModel::BaseWorldModel()
{
//	BaseWorldModel::count++;
//	ENSURE(count == 1);
		
	setOurScore(0);
	setOtherScore(0);
	pwpOurGoalie = NULL;
	pwpOtherGoalie = NULL;

	pcTeamName = NULL;
	
	for(int i = 0; i < MAX_PLAYER; i++)
	{		
		pwpOurTeam[i] = new WorldPlayer();		
		pwpOtherTeam[i] = new WorldPlayer();
		
		// TODO: Should this be someplace else? JM
		pwpOurTeam[i]->setTeam(TEAM_US);
		pwpOtherTeam[i]->setTeam(TEAM_THEM);
	}		


  // TODO: set goal's (and all other world objects) absolute position
  //			 keep an array of flags indexed by type with each flags absolute
  //			 position (therefore deprecating global FLAGPOS function)
  // WARNING: hmmm... do goal left and goal right need to change based on playing side?
  woOurGoal.setAbsPosition(GOALPOS(GOAL_LEFT));
  woOtherGoal.setAbsPosition(GOALPOS(GOAL_RIGHT));
}


BaseWorldModel::~BaseWorldModel()
{
    delete pcTeamName;
		for(int i = 0; i < MAX_PLAYER; i++)
		{
			delete pwpOurTeam[i];
			delete pwpOtherTeam[i];
		}		
}

void BaseWorldModel::updateWorldPlayer(const WorldPlayer& _wpPlayer)
{
    ENSURE(_wpPlayer.getTeam() != TEAM_UNKNOWN);

    int iIndex = _wpPlayer.getNumber()-1; // Minus 1!!!

    WorldPlayer** theTeam;
    if (_wpPlayer.getTeam() == TEAM_US )
        theTeam = pwpOurTeam;
    else
        theTeam = pwpOtherTeam;

    // TODO: Make sure I didn't forget anything
    theTeam[iIndex]->setAbsPosition(_wpPlayer.getAbsPosition());
    theTeam[iIndex]->setVelocity(_wpPlayer.getVelocity());
    theTeam[iIndex]->setAge(_wpPlayer.getAge());
    theTeam[iIndex]->setTeam(_wpPlayer.getTeam());
    theTeam[iIndex]->setNumber(_wpPlayer.getNumber());
    theTeam[iIndex]->setHeadDir(_wpPlayer.getHeadDir());
    theTeam[iIndex]->setBodyDir(_wpPlayer.getBodyDir());
}

WorldBall BaseWorldModel::getWorldBall() const
{
  return wbBall;
}
	
WorldPlayer BaseWorldModel::getOurPlayer(int _iPlayerNum) const
{
  ENSURE((_iPlayerNum >= 1) && (_iPlayerNum <= MAX_PLAYER));
  return *pwpOurTeam[_iPlayerNum - 1];
}

WorldPlayer BaseWorldModel::getOtherPlayer(int _iPlayerNum) const
{
  ENSURE((_iPlayerNum >= 1) && (_iPlayerNum <= MAX_PLAYER));
  return *pwpOtherTeam[_iPlayerNum - 1];
}

WorldPlayer BaseWorldModel::getOurGoalie() const
{
    for(int i = 1; i <= MAX_PLAYER; ++i)
    {
        if (getOurPlayer(i).isGoalie())
            return getOurPlayer(i);
    }

    return WorldPlayer();
//	if (pwpOurGoalie != NULL);
//	    return *pwpOurGoalie;
//
//	return WorldPlayer();  // goalie is unknown, return an unknown player
}

void BaseWorldModel::setOurGoalie (WorldPlayer* _pwpGoalie)
{
	pwpOurGoalie = _pwpGoalie;
}

WorldPlayer BaseWorldModel::getOtherGoalie() const
{
	if (pwpOtherGoalie != NULL);
	    return *pwpOtherGoalie;

	return WorldPlayer();  // goalie is unknown, return an unknown player
}

void BaseWorldModel::setOtherGoalie (WorldPlayer* _pwpGoalie)
{
	pwpOtherGoalie = _pwpGoalie;
}

/*
void BaseWorldModel::setOurGoal(goal_type _gtWhichGoal)
{
	// TODO: perhaps, add += operator to worldobject for consistency
  woOurGoal.setAbsPosition(GOALPOS(_gtWhichGoal));
}

void BaseWorldModel::setOtherGoal(goal_type _gtWhichGoal)
{
	// TODO: perhaps, add += operator to worldobject for consistency
  woOtherGoal.setAbsPosition(GOALPOS(_gtWhichGoal));
}
*/

WorldObject BaseWorldModel::getOurGoal() const
{
  return woOurGoal;
}

WorldObject BaseWorldModel::getOtherGoal() const
{
  return woOtherGoal;
}


void BaseWorldModel::setPlayMode(play_mode _pmPlayMode)
{
  INSPECT(getTime() << ": Setting playmode " << _pmPlayMode);
  pmPlayMode = _pmPlayMode;
}

play_mode BaseWorldModel::getPlayMode() const
{
  return pmPlayMode;
}
	
void BaseWorldModel::setOurScore(int _iOurScore)
{
  iOurScore = _iOurScore;
}
	
int BaseWorldModel::getOurScore() const
{
  return iOurScore;
}
	
void BaseWorldModel::setOtherScore(int _iOtherScore)
{
  iOtherScore = _iOtherScore;
}
	
int BaseWorldModel::getOtherScore() const
{
  return iOtherScore;
}
	
void BaseWorldModel::setTime(int _iTime)
{
    if(_iTime != iTime) age();


//    for(int i=0; i < (_iTime-iTime); ++i)
//        age(); // age the worldmodel

    iTime = _iTime;
    LowLevelComm::iWorldTime = _iTime; // ugly global
}
	
int BaseWorldModel::getTime() const
{
  return iTime;
}

void BaseWorldModel::setTeamName(char* _pcTeamName)
{
  if(pcTeamName != NULL) delete pcTeamName;
  pcTeamName = new char[strlen(_pcTeamName)+1];
  strcpy(pcTeamName, _pcTeamName);
}

char* BaseWorldModel::getTeamName() const
{
  return pcTeamName;
}


bool BaseWorldModel::inSecondHalf() const
{
	return (getTime() > HALF_TIME);
}

bool BaseWorldModel::isTimeRunning() const
{
	// TODO: investigate play mode and return
	//			 true if the play mode is a mode where
	//			 time stops
	return true;
}

void BaseWorldModel::age()
{
    wbBall.incrementAge();
		
    for(int i=0; i < MAX_PLAYER; ++i)
    {
        pwpOtherTeam[i]->incrementAge();
        pwpOurTeam[i]->incrementAge();
    }
}

ofstream& BaseWorldModel::ofstream_insert(ofstream& _out)
{
    _out << "<baseworldmodel time=\"" << getTime() << "\">" << endl;
//    _out << "<time>" << getTime() << "</time>" << endl;

    // TODO (optional): Make a ball special, and not just a worldmovable object
    _out << "  <ball>" << endl;
    getWorldBall().ofstream_insert(_out);
    _out << "  </ball>" << endl;

    // Log all active players
    for(int i=0; i < MAX_PLAYER; ++i) {
        if(pwpOurTeam[i] && (pwpOurTeam[i]->getAge() != UNKNOWN)) {
            pwpOurTeam[i]->ofstream_insert(_out);
        }
    }
    for(int i=0; i < MAX_PLAYER; ++i) {
        if(pwpOtherTeam[i] && (pwpOtherTeam[i]->getAge() != UNKNOWN)) {
            pwpOtherTeam[i]->ofstream_insert(_out);
        }
    }

    // Log goals
    _out << "  <ourgoal absx=\"" << getOurGoal().getAbsPosition().getX()
         << "\" absy=\"" << getOurGoal().getAbsPosition().getY() << "\"/>" << endl;

    _out << "  <othergoal absx=\"" << getOtherGoal().getAbsPosition().getX()
         << "\" absy=\"" << getOtherGoal().getAbsPosition().getY() << "\"/>" << endl;

    _out << "</baseworldmodel>" << endl;

    return _out;
}

ifstream& BaseWorldModel::ifstream_extract(ifstream &_in) {
    // TODO: Something

    return _in;
}

