/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// coachworldmodel.cpp Thu Jun 28 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "coachworldmodel.h"

CoachWorldModel::CoachWorldModel()
{
}

CoachWorldModel::~CoachWorldModel()
{
}

void CoachWorldModel::setOurGoal(const ExactGoal &_egOurGoal)
{
    woOurGoal += _egOurGoal;
}

void CoachWorldModel::setOtherGoal(const ExactGoal &_egOtherGoal)
{
    woOtherGoal += _egOtherGoal;
}

void CoachWorldModel::updateWorldBall(const ExactObject &_eoBall)
{
    wbBall += _eoBall;
}

// updatePlayer() may be combined into an overloaded updatePlayer() if it gets too large.
// This one is called with ExactPlayers, which are just typedef's of SensorPlayers.
// That would have to change to do the overloading
void CoachWorldModel::updatePlayer(const ExactPlayer& _ePlayer)
{
    WorldPlayer *pwpThisPlayer = NULL;
    if(!_ePlayer.valid) return;
	if (!((_ePlayer.number >= 1) && (_ePlayer.number <= MAX_PLAYER)))
	{
	    TRACE( "Error!  Valid player with invalid number received!!!" << endl );
	    return;
	}
	
    if (_ePlayer.team == TEAM_US) {
        pwpThisPlayer = pwpOurTeam[_ePlayer.number - 1];
        if(pwpThisPlayer->isGoalie()) setOurGoalie(pwpThisPlayer);
    }
    else if (_ePlayer.team == TEAM_THEM)
    {
        pwpThisPlayer = pwpOtherTeam[_ePlayer.number - 1];
        if(pwpThisPlayer->isGoalie()) setOtherGoalie(pwpThisPlayer);
    }
    else
    {
        TRACE( "Error! Valid player with valid number and invalid team received!" << endl );
        return;
    }

    pwpThisPlayer->setAge(0);
    pwpThisPlayer->setBodyDir(_ePlayer.body_dir);
    pwpThisPlayer->setHeadDir(_ePlayer.head_dir);
    pwpThisPlayer->setGoalie(_ePlayer.goalie == GOALIE_TRUE);
    pwpThisPlayer->setAbsPosition(Vector2d(_ePlayer.x ,_ePlayer.y));
    pwpThisPlayer->setVelocity(Vector2d(_ePlayer.x_vel, _ePlayer.y_vel));
    pwpThisPlayer->setNumber(_ePlayer.number);


/*

  if(_ePlayer.valid)
  {		
		// if the player is on our team
		if (_ePlayer.team == TEAM_US)
		{
			// and the player's number is known
			if ((_ePlayer.number >= 1) && (_ePlayer.number <= MAX_PLAYER))
			{
				// copy the sensor player into the proper WorldPlayer object
				*pwpOurTeam[_ePlayer.number - 1] += _ePlayer;
				
				// if we don't know who the goalie is...
				if (pwpOurGoalie == NULL)
					// and if this player is a goalie, keep a pointer for quick reference
    				if (pwpOurTeam[_ePlayer.number - 1]->isGoalie()) setOurGoalie(pwpOurTeam[_ePlayer.number - 1]);
			}
			else
			{
				// otherwise, we have an error
				TRACE("A player on our team exists with an unknown number " << _ePlayer.number << ".");
			}
		}
		else if (_ePlayer.team == TEAM_THEM)
		{
			if ((_ePlayer.number >= 1) && (_ePlayer.number <= MAX_PLAYER))
			{
				// set the WorldPlayer equal to the SensorPlayer (overloaded operator= in WorldPlayer)
				*pwpOtherTeam[_ePlayer.number - 1] += _ePlayer;
				
	
				// if we don't know who the goalie is...
				if (pwpOtherGoalie == NULL)
					// keep a pointer to the goalie for quick reference
					if (pwpOtherTeam[_ePlayer.number-1]->isGoalie()) setOtherGoalie(pwpOtherTeam[_ePlayer.number-1]);
			}
			else
			{
				TRACE("A player on their team exists with an unknown number: " << _ePlayer.number << ".");
			}
		}
		else
		{
			TRACE("A player exists with number " << _ePlayer.number << " that does not belong to either team.");
		}
	} // end if(valid)
	
*/
}


/*
ofstream& CoachWorldModel::ofstream_insert(ofstream& _out)
{
    _out << "\niTime: " << getTime() << endl;

    for(int i=0; i<11; i++)
    {
        if(pwpOurTeam[i] && pwpOurTeam[i]->getAge() != ETERNAL) pwpOurTeam[i]->ofstream_insert(_out);
    }

    for(int i=0; i<11; i++)
    {
        if(pwpOtherTeam[i] && pwpOtherTeam[i]->getAge() != ETERNAL) pwpOtherTeam[i]->ofstream_insert(_out);
    }

  return _out;
}
*/
/*
ifstream& CoachWorldModel::ifstream_extract(ifstream& _in)
{
  if( !_in )
  {
    cerr << "WorldModel: input stream not open or faulty when trying to read"
    	   << endl;
    return _in;
  }

  return _in;
}
*/

// Read binary data from log file
ifstream& operator>>(ifstream& _in, CoachWorldModel &_cwmWorld)
{
  return _cwmWorld.ifstream_extract(_in);
}

// WARNING: this file dumps binary data!!!
ofstream& operator<<(ofstream& _out, CoachWorldModel &_cwmWorld)
{
  return _cwmWorld.ofstream_insert(_out);
}

