/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// worldplayer.cpp Fri Jun 8 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "worldplayer.h"

WorldPlayer::WorldPlayer() : WorldMovableObject(PLAYER_SPEED_MAX, PLAYER_SIZE)
{
	setNumber(UNKNOWN);
	setTeam(TEAM_UNKNOWN);
	setGoalie(false);
	setBodyDir(Angle(0));
	setHeadDir(Angle(0));
	setEffort(UNKNOWN);
	setStamina(UNKNOWN);
}

WorldPlayer::WorldPlayer(const Vector2d& _vAbsPosition) : WorldMovableObject(PLAYER_SPEED_MAX, PLAYER_SIZE, _vAbsPosition)
{
	setNumber(UNKNOWN);
	setTeam(TEAM_UNKNOWN);
	setGoalie(false);
	setBodyDir(Angle(0));
	setHeadDir(Angle(0));
	setEffort(UNKNOWN);
	setStamina(UNKNOWN);
}

WorldPlayer::WorldPlayer(int _iNumber, team_number _tnTeam, bool _fGoalie):
	WorldMovableObject(PLAYER_SPEED_MAX, PLAYER_SIZE)
{
	setNumber ( _iNumber );
	setTeam ( _tnTeam );
	setGoalie ( _fGoalie );
	setBodyDir ( Angle(0) );
	setHeadDir ( Angle(0) );
}

WorldPlayer::~WorldPlayer()
{

}

Angle WorldPlayer::getBodyDir () const
{
	return aBodyDir;
}

Angle WorldPlayer::getHeadDir () const
{
	return aHeadDir;
}

Angle WorldPlayer::getFaceDir () const
{
	return (getBodyDir() + getHeadDir());
}

int WorldPlayer::getNumber() const
{
	return iNumber;
}

bool WorldPlayer::isGoalie() const
{
	return fGoalie;
}

team_number WorldPlayer::getTeam() const
{
	return tnTeam;
}

void WorldPlayer::setBodyDir (Angle _aBodyDir)
{
	aBodyDir = _aBodyDir;
}

void WorldPlayer::setHeadDir (Angle _aHeadDir)
{
	aHeadDir = _aHeadDir;
}

void WorldPlayer::setNumber (int _iNumber)
{
	iNumber = _iNumber;
}

void WorldPlayer::setGoalie (bool _fGoalie)
{
	fGoalie = _fGoalie;
}

bool WorldPlayer::isYoung(int _iAge)
{
 if ( (isKnown() == true) && (getAge() < _iAge))
 	return true;
 	
 return false;
		
}

void WorldPlayer::setTeam (team_number _tnTeam)
{
	tnTeam = _tnTeam;
}

void WorldPlayer::setStamina(const double &_dStamina)
{
  dStamina = _dStamina;
}


double WorldPlayer::getStamina() const
{
	return dStamina;
}


void WorldPlayer::setEffort(const double &_dEffort)
{
	dEffort = _dEffort;
}

double WorldPlayer::getEffort() const
{
	return dEffort;
}

bool WorldPlayer::operator==(const WorldPlayer& _wpPlayer) const
{
    return (getNumber() == _wpPlayer.getNumber()) && (getTeam() == _wpPlayer.getTeam());
}


/*
WorldPlayer& WorldPlayer::operator+=(const SensorPlayer &spPlayer)
{
	// NOTE: this function does not set the WorldPlayer's absolute
	//			 position.
	// TODO: fix the fact that we're setting relative information
	//			 in this function, perhaps deprecate completely and
	//			 set all information in worldmodel

  // if we saw the player
	if (spPlayer.valid)
	{
		// first adjust the age and conetime
		setAge(0);
		setKnown(); // will result in extra calls but can't figure out a better way
		increaseConeTime();
		
		// then update the velocity		
		//setVelocity(spPlayer.dir_change, spPlayer.dist_change);
	
		// set body and face direction
		setBodyDir(spPlayer.body_dir);
		setHeadDir(spPlayer.head_dir);
		
		// set the players jersey number
		setNumber(spPlayer.number);
		
		// is the player the goalie?
		if (spPlayer.goalie == GOALIE_TRUE)
			setGoalie(true);
		else
			setGoalie(false);
	}
		
	// set the cone time to zero and increase the age
	else if(isKnown())
	{
	    incrementAge();
		if (getAge() == 0)
			setConeTime(0);
	}	
	
	return *this;
}


WorldPlayer& WorldPlayer::operator+=(const ExactPlayer &_ePlayer)
{
  if(_ePlayer.valid)
  {
    setAge(0);
    setBodyDir(_ePlayer.body_dir);
    setHeadDir(_ePlayer.head_dir);
    setGoalie(_ePlayer.goalie == GOALIE_TRUE);
    setAbsPosition(Vector2d(_ePlayer.x ,_ePlayer.y));
    setVelocity(Vector2d(_ePlayer.x_vel, _ePlayer.y_vel));
    setNumber(_ePlayer.number);
  }

    return *this;
}
*/


ofstream& WorldPlayer::ofstream_insert(ofstream& _fout)
{
    _fout << "  <worldplayer team=\"" << getTeam() << "\" number=\"" << getNumber()
         << "\" headdir=\"" << getHeadDir() << "\" bodydir=\"" << getBodyDir() << "\">" << endl;

    WorldMovableObject::ofstream_insert(_fout);

    _fout << "  </worldplayer>" << endl;


    return _fout;
}

ostream& WorldPlayer::ostream_insert(ostream& _out)
{
    _out << "  <worldplayer team=\"" << getTeam() << "\" number=\"" << getNumber()
         << "\" headdir=\"" << getHeadDir() << "\" bodydir=\"" << getBodyDir() << "\">" << endl;

    WorldMovableObject::ostream_insert(_out);

    _out << "  </worldplayer>" << endl;


    return _out;
}


ifstream& WorldPlayer::ifstream_extract(ifstream& _in)
{
    // TODO: Something

    return _in;
}

ostream& operator<<(ostream& _out, WorldPlayer &_wpPlayer)
{
  return _wpPlayer.ostream_insert(_out);
}

